#include <windows.h>
#include "dinput.h"
#include "main.h"
#include "input.h"
#include "registry.h"

REGISTRY_INFO								RegistryInfo;

// Sets up the default registry entries, either for first time use
// or incase the load registry entries fails
void InitRegistryEntries(void)
{
	unsigned char	ButtonMap[NO_OF_BUTTONS] = REGISTRY_DEFAULT_BUTTON_MAP;
	char			TempPath[MAX_PATH];

	GetTempPath(MAX_PATH, TempPath);

	memset(&RegistryInfo, 0, sizeof(REGISTRY_INFO));

	RegistryInfo.ScreenScale	 = REGISTRY_DEFAULT_SCREEN_SCALE;
	RegistryInfo.UpdateFrequency = REGISTRY_DEFAULT_UPDATE_FREQUENCY;
	RegistryInfo.FrameSkip		 = REGISTRY_DEFAULT_FRAME_SKIP;
	memcpy(RegistryInfo.ButtonMap, ButtonMap, sizeof(ButtonMap));
	strcpy(RegistryInfo.ROMPath, REGISTRY_DEFAULT_ROM_PATH);
	strcpy(RegistryInfo.SRAMPath, TempPath);
}

// Loads the settings from the registry
BOOL LoadRegistryEntries(void)
{
	char			Key[_MAX_PATH];
	int				Result;
	HKEY			hkey;
	unsigned long	DataSize;

	wsprintf(Key, "Software%s", REGISTRY_KEY);
	
	Result = RegOpenKeyEx(HKEY_CURRENT_USER,	// handle
						  Key,					// subkey
						  0,
						  KEY_READ,				// access mask
						  &hkey);				// pointer to recieving handle

	if (Result != ERROR_SUCCESS) return FALSE;

	DataSize = sizeof(RegistryInfo.ScreenScale);
	RegQueryValueEx(hkey, REGISTRY_SCREEN_SCALE, NULL, NULL, (LPBYTE)&RegistryInfo.ScreenScale, &DataSize);
	DataSize = sizeof(RegistryInfo.UpdateFrequency);
	RegQueryValueEx(hkey, REGISTRY_UPDATE_FREQUENCY, NULL, NULL, (LPBYTE)&RegistryInfo.UpdateFrequency, &DataSize);
	DataSize = sizeof(RegistryInfo.FrameSkip);
	RegQueryValueEx(hkey, REGISTRY_FRAME_SKIP, NULL, NULL, (LPBYTE)&RegistryInfo.FrameSkip, &DataSize);
	DataSize = sizeof(RegistryInfo.ButtonMap);
	RegQueryValueEx(hkey, REGISTRY_BUTTON_MAP, NULL, NULL, (LPBYTE)&RegistryInfo.ButtonMap, &DataSize);
	DataSize = sizeof(RegistryInfo.ROMPath);
	RegQueryValueEx(hkey, REGISTRY_ROM_PATH, NULL, NULL, (LPBYTE)&RegistryInfo.ROMPath, &DataSize);
	DataSize = sizeof(RegistryInfo.SRAMPath);
	RegQueryValueEx(hkey, REGISTRY_SRAM_PATH, NULL, NULL, (LPBYTE)&RegistryInfo.SRAMPath, &DataSize);

	RegCloseKey(hkey);			

	return TRUE;
}

// Saves the settings to the registry
BOOL SaveRegistryEntries(void)
{
	char			Key[_MAX_PATH];
	HKEY			hkey;
	DWORD			dwdisp;
	int				Result;

	wsprintf(Key, "Software%s", REGISTRY_KEY);

	Result = RegCreateKeyEx(HKEY_CURRENT_USER,			// handle
							Key,						// subkey
							0,							// reserved
							"",							// class string
							REG_OPTION_NON_VOLATILE,	// flags
							KEY_ALL_ACCESS,				// access mask	
							NULL,						// security attributes ptr
							&hkey,						// pointer to recieving handle
							&dwdisp);					// pointer to recieving disposition

	if (Result != ERROR_SUCCESS) return FALSE;

	RegSetValueEx(hkey, REGISTRY_SCREEN_SCALE, 0, REG_BINARY, (BYTE *)&RegistryInfo.ScreenScale, sizeof(RegistryInfo.ScreenScale));
	RegSetValueEx(hkey, REGISTRY_UPDATE_FREQUENCY, 0, REG_BINARY, (BYTE *)&RegistryInfo.UpdateFrequency, sizeof(RegistryInfo.UpdateFrequency));
	RegSetValueEx(hkey, REGISTRY_FRAME_SKIP, 0, REG_BINARY, (BYTE *)&RegistryInfo.FrameSkip, sizeof(RegistryInfo.FrameSkip));
	RegSetValueEx(hkey, REGISTRY_BUTTON_MAP, 0, REG_BINARY, (BYTE *)&RegistryInfo.ButtonMap, sizeof(RegistryInfo.ButtonMap));
	RegSetValueEx(hkey, REGISTRY_ROM_PATH, 0, REG_SZ, (BYTE *)RegistryInfo.ROMPath, strlen(RegistryInfo.ROMPath) + 1);
	RegSetValueEx(hkey, REGISTRY_SRAM_PATH, 0, REG_SZ, (BYTE *)RegistryInfo.SRAMPath, strlen(RegistryInfo.SRAMPath) + 1);
	
	RegCloseKey(hkey);

	return TRUE;
}

void SetFileAssociations(void)
{
	HKEY			hkey;
	DWORD			dwdisp;
	char			Filename[MAX_PATH];
	char			String[MAX_PATH];

	GetModuleFileName(GetModuleHandle(NULL), Filename, MAX_PATH);

	RegDeleteKey(HKEY_CLASSES_ROOT, ".sms");
	RegDeleteKey(HKEY_CLASSES_ROOT, "smsfile");
	RegDeleteKey(HKEY_CLASSES_ROOT, ".gg");
	RegDeleteKey(HKEY_CLASSES_ROOT, "ggfile");

	strcpy(String, "smsfile");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, ".sms", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	strcpy(String, "MasterSystem");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "smsfile", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	strcpy(String, "");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "smsfile\\Shell", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	String[0] = '"';
	memcpy(&String[1], Filename, strlen(Filename) + 1);
	strcat(String, "\" %1");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "smsfile\\Shell\\Emulate\\command", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	memcpy(String, Filename, strlen(Filename) + 1);
	strcat(String, ",0");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "smsfile\\DefaultIcon", 0, "",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	strcpy(String, "ggfile");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, ".gg", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	strcpy(String, "GameGear");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "ggfile", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	strcpy(String, "");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "ggfile\\Shell", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	String[0] = '"';
	memcpy(&String[1], Filename, strlen(Filename) + 1);
	strcat(String, "\" %1");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "ggfile\\Shell\\Emulate\\command", 0,	"",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);

	memcpy(String, Filename, strlen(Filename) + 1);
	strcat(String, ",0");
	RegCreateKeyEx(HKEY_CLASSES_ROOT, "ggfile\\DefaultIcon", 0, "",	REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, &dwdisp);
	RegSetValueEx(hkey, "", 0, REG_SZ, (BYTE *)String, strlen(String) + 1);
	RegCloseKey(hkey);
}
